@@ -37,7 +37,8 @@ enum icu_chain_step_type {
     ICU_chain_step_type_transform,      /* apply utf16 transform */
     ICU_chain_step_type_tokenize,       /* apply utf16 tokenization */
     ICU_chain_step_type_transliterate,  /* apply utf16 tokenization */
-    YAZ_chain_step_type_stemming        /* apply utf16 stemming (YAZ) */
+    YAZ_chain_step_type_stemming,       /* apply utf16 stemming (YAZ) */
+    ICU_chain_step_type_join
 };
 
 struct icu_chain_step
@@ -49,6 +50,7 @@ struct icu_chain_step
 	struct icu_transform *transform;
 	struct icu_tokenizer *tokenizer;
         yaz_stemmer_p         stemmer;
+        struct icu_buf_utf16 *join;
     } u;
     struct icu_chain_step *previous;
 };
@@ -77,41 +79,46 @@ int icu_check_status(UErrorCode status)
 
 static struct icu_chain_step *icu_chain_insert_step(
     struct icu_chain *chain, enum icu_chain_step_type type,
-    const uint8_t *rule, UErrorCode *status)
+    const char *rule, UErrorCode *status)
 {
     struct icu_chain_step *step = 0;
 
-    if (!chain || !type || !rule)
-        return 0;
+    assert(chain);
+    assert(type);
 
     step = (struct icu_chain_step *) xmalloc(sizeof(*step));
-
     step->type = type;
-    /* create auxilary objects */
+
     switch (step->type)
     {
     case ICU_chain_step_type_display:
         break;
     case ICU_chain_step_type_casemap:
+        assert(rule);
         step->u.casemap = icu_casemap_create(rule[0], status);
         break;
     case ICU_chain_step_type_transform:
+        assert(rule);
         /* rule omitted. Only ID used */
-        step->u.transform = icu_transform_create((const char *) rule, 'f',
-                                                 0, status);
+        step->u.transform = icu_transform_create(rule, 'f', 0, status);
         break;
     case ICU_chain_step_type_tokenize:
-        step->u.tokenizer = icu_tokenizer_create(chain->locale,
-                                                 (char) rule[0], status);
+        assert(rule);
+        step->u.tokenizer = icu_tokenizer_create(chain->locale, rule[0], status);
         break;
     case ICU_chain_step_type_transliterate:
+        assert(rule);
         /* we pass a dummy ID to utrans_openU.. */
-        step->u.transform = icu_transform_create("custom", 'f',
-                                                 (const char *) rule, status);
+        step->u.transform = icu_transform_create("custom", 'f', rule, status);
         break;
     case YAZ_chain_step_type_stemming:
-        step->u.stemmer = yaz_stemmer_create(chain->locale,
-                                             (const char *) rule, status);
+        assert(rule);
+        step->u.stemmer = yaz_stemmer_create(chain->locale, rule, status);
+        break;
+    case ICU_chain_step_type_join:
+        assert(rule);
+        step->u.join = icu_buf_utf16_create(0);
+        icu_utf16_from_utf8_cstr(step->u.join, rule, status);
         break;
     default:
         break;
@@ -147,6 +154,9 @@ static void icu_chain_step_destroy(struct icu_chain_step *step)
     case YAZ_chain_step_type_stemming:
         yaz_stemmer_destroy(step->u.stemmer);
         break;
+    case ICU_chain_step_type_join:
+        icu_buf_utf16_destroy(step->u.join);
+        break;
     default:
         break;
     }
@@ -181,6 +191,10 @@ struct icu_chain_step *icu_chain_step_clone(struct icu_chain_step *old)
             break;
         case ICU_chain_step_type_none:
             break;
+        case ICU_chain_step_type_join:
+            (*sp)->u.join = icu_buf_utf16_create(0);
+            (*sp)->u.join = icu_buf_utf16_copy((*sp)->u.join, old->u.join);
+            break;
         }
         old = old->previous;
         sp = &(*sp)->previous;
@@ -270,36 +284,48 @@ struct icu_chain *icu_chain_xml_config(const xmlNode *xml_node,
                 yaz_log(YLOG_WARN, "Unsupported attribute '%s' for "
                         "element '%s'", attr->name, node->name);
                 no_errors++;
-                continue;
             }
         }
         if (!rule && node->children)
             rule = nmem_text_node_cdata(node->children, nmem);
 
+        if (!rule && strcmp((const char *) node->name, "display"))
+        {
+            yaz_log(YLOG_WARN, "Missing attribute rule for element %s",
+                    (const char *) node->name);
+            no_errors++;
+            continue;
+        }
         if (!strcmp((const char *) node->name, "casemap"))
-            step = icu_chain_insert_step(chain, ICU_chain_step_type_casemap,
-                                         (const uint8_t *) rule, status);
+            step = icu_chain_insert_step(chain,
+                                         ICU_chain_step_type_casemap,
+                                         rule, status);
         else if (!strcmp((const char *) node->name, "transform"))
-            step = icu_chain_insert_step(chain, ICU_chain_step_type_transform,
-                                         (const uint8_t *) rule, status);
+            step = icu_chain_insert_step(chain,
+                                         ICU_chain_step_type_transform,
+                                         rule, status);
         else if (!strcmp((const char *) node->name, "transliterate"))
-            step = icu_chain_insert_step(chain, ICU_chain_step_type_transliterate,
-                                         (const uint8_t *) rule, status);
+            step = icu_chain_insert_step(chain,
+                                         ICU_chain_step_type_transliterate,
+                                         rule, status);
         else if (!strcmp((const char *) node->name, "tokenize"))
             step = icu_chain_insert_step(chain, ICU_chain_step_type_tokenize,
-                                         (const uint8_t *) rule, status);
+                                         rule, status);
         else if (!strcmp((const char *) node->name, "display"))
             step = icu_chain_insert_step(chain, ICU_chain_step_type_display,
-                                         (const uint8_t *) "", status);
+                                         rule, status);
         else if (!strcmp((const char *) node->name, "stemming"))
             step = icu_chain_insert_step(chain, YAZ_chain_step_type_stemming,
-                                         (const uint8_t *) rule, status);
+                                         rule, status);
+        else if (!strcmp((const char *) node->name, "join"))
+            step = icu_chain_insert_step(chain, ICU_chain_step_type_join,
+                                         rule, status);
         else if (!strcmp((const char *) node->name, "normalize"))
         {
             yaz_log(YLOG_WARN, "Element %s is deprecated. "
                     "Use transform instead", node->name);
             step = icu_chain_insert_step(chain, ICU_chain_step_type_transform,
-                                         (const uint8_t *) rule, status);
+                                         rule, status);
         }
         else if (!strcmp((const char *) node->name, "index")
                  || !strcmp((const char *) node->name, "sortkey"))
@@ -313,6 +339,11 @@ struct icu_chain *icu_chain_xml_config(const xmlNode *xml_node,
             no_errors++;
             continue;
         }
+        if (!step)
+        {
+            yaz_log(YLOG_WARN, "Step not created for %s", node->name);
+            no_errors++;
+        }
         if (step && U_FAILURE(*status))
         {
             no_errors++;
@@ -347,7 +378,6 @@ void icu_utf16_print(struct icu_buf_utf16 *src16)
     struct icu_buf_utf8 *dst8 = icu_buf_utf8_create(0);
     icu_utf16_to_utf8(dst8, src16, &status);
 
-    assert(status != 1234);
     if (U_FAILURE(status))
     {
         printf("failure");
@@ -422,6 +452,22 @@ struct icu_buf_utf16 *icu_iter_invoke(yaz_icu_iter_t iter,
                 icu_buf_utf16_destroy(src);
             }
             break;
+        case ICU_chain_step_type_join:
+            if (dst)
+            {
+                while (1)
+                {
+                    struct icu_buf_utf16 *dst1 =
+                        icu_iter_invoke(iter, step->previous, 0);
+
+                    if (!dst1)
+                        break; 
+                    dst = icu_buf_utf16_append(dst, step->u.join);
+                    dst = icu_buf_utf16_append(dst, dst1);
+                    icu_buf_utf16_destroy(dst1);
+                }
+            }
+            break;
         default:
             assert(0);
         }
@@ -476,9 +522,8 @@ int icu_iter_next(yaz_icu_iter_t iter)
     else
     {
         /* on first call, iter->input is the input string. Thereafter: 0. */
-        iter->last = icu_iter_invoke(iter, iter->steps ?
-                                     iter->steps : iter->chain->csteps,
-                                     iter->input);
+        assert(iter->steps || !iter->chain->csteps);
+        iter->last = icu_iter_invoke(iter, iter->steps, iter->input);
         iter->input = 0;
 
         if (!iter->last)
